package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.feign.IMFeign;
import com.steerinfo.dil.feign.TmsTruckFeign;
import com.steerinfo.dil.mapper.OmstruckOrderMapper;
import com.steerinfo.dil.mapper.OmstruckOrderMaterialMapper;
import com.steerinfo.dil.mapper.OmstruckOrderSeparateMapper;
import com.steerinfo.dil.model.OmstruckOrder;
import com.steerinfo.dil.model.OmstruckOrderMaterial;
import com.steerinfo.dil.service.IOmstruckOrderSeparateService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;

/**
 * 运输订单第二个 impl
 * @ author    :TXF
 * @ time      :2021/12/3 13:14
 */

@Service
public class OmstruckOrderSeparateServiceImpl implements IOmstruckOrderSeparateService {

    @Autowired
    private OmstruckOrderSeparateMapper omstruckOrderSeparateMapper;

    @Autowired
    private OmstruckOrderMapper omstruckOrderMapper;

    @Autowired
    private OmstruckOrderServiceImpl omstruckOrderService;

    @Autowired
    private TmsTruckFeign tmsTruckFeign;

    @Autowired
    OmstruckOrderMaterialMapper omstruckOrderMaterialMapper;

    @Autowired
    IMFeign imFeign;

    //通过线路子表路线图生成各实绩
    public int addAllSonResult(Integer resultTotalId, Integer lineId){
        int result = 0;
        //通过订单Id 查询关联的线路子表顺序
        BigDecimal orderId = omstruckOrderSeparateMapper.getOrderIdByTotalId(new BigDecimal(resultTotalId));
        List<Map<String, Object>> segmentList = omstruckOrderSeparateMapper.getLineMesByOrderId(lineId);
        Map<String,Object> totalIdMap = new HashMap<>();
        totalIdMap.put("resultTotalId",resultTotalId);
        // 遍历路段顺序号子表
        int count = 0;
        BigDecimal lineType = (BigDecimal) segmentList.get(0).get("lineType");
        if (lineType.intValue() == 4) {
            count++;
        }
        for (Map<String,Object> segmentMap : segmentList) {
            BigDecimal segmentSqe = (BigDecimal) segmentMap.get("segmentSqe");
//            BigDecimal type = (BigDecimal) segmentMap.get("type");
            String linkName = (String) segmentMap.get("linkName");
            totalIdMap.put("segmentSqe",segmentSqe);
            // 判断是否是计量
            if (linkName.equals("计毛") || linkName.equals("计皮")) {
                // 如果是计量则加一
                count ++;
                // 有两个计量的时候则新增实绩
                if (count == 2) {
                    tmsTruckFeign.addWeightResult(totalIdMap);
                    // 新增一次则对计数器清零
                    count = 0;
                    result++;
                }
            }
            // 进厂
            if (linkName.equals("进厂")) {
                tmsTruckFeign.addEnFactoryResult(totalIdMap);
                result++;
            }
            // 出厂
            if (linkName.equals("出厂")) {
                tmsTruckFeign.addLeaveFactory(totalIdMap);
                result++;
            }
            // 装货
            if (linkName.equals("装货")) {
                tmsTruckFeign.addLoadResult(totalIdMap);
                result++;
            }
            // 卸货
            if (linkName.equals("卸货")) {
                tmsTruckFeign.addUnloadResult(totalIdMap);
                result++;
            }
            // 退货
            if (linkName.equals("退货")) {
                tmsTruckFeign.addReturnGoodsResult(totalIdMap);
                result++;
            }
        }
        return result;
    }


    /**
     * 添加退货实绩关闭当前订单
     * @param map
     * @return
     */
    public int returnOrderCloseOrder(Map<String, Object> map){
        //通过运输订单ID关闭运输订单(修改状态为退货关闭：9)
        map.put("orderStatus", 9);
        int i = omstruckOrderSeparateMapper.updateOrderStatusByOrderNum(map);
        BigDecimal orderId = DataChange.dataToBigDecimal(map.get("orderId"));
        //查询之前订单所有的信息
        Map<String, Object> orderMap = omstruckOrderSeparateMapper.getOmstruckOrderResult(orderId);
        // 获得运单信息
        Map<String,Object> orderMessaggeMap = omstruckOrderSeparateMapper.getOrderMessagge(orderId);
        //添加新运输订单 运输订单号
        OmstruckOrder omstruckOrder = new OmstruckOrder();
        BigDecimal newOrderId = omstruckOrderMapper.selectMaxId();
        // 卸货点
        BigDecimal unloadPointId = (BigDecimal) orderMessaggeMap.get("unloadPointId");
        omstruckOrder.setOrderId(newOrderId);
        omstruckOrder.setOrderType(DataChange.dataToBigDecimal(orderMessaggeMap.get("orderType")));
        omstruckOrder.setLineId(DataChange.dataToBigDecimal(map.get("lineId")));
        omstruckOrder.setOrderPlanId(DataChange.dataToBigDecimal(orderMessaggeMap.get("orderPlanId")));
        omstruckOrder.setOrderStatus(new BigDecimal(1)); //执行中
        omstruckOrder.setCapacityId(DataChange.dataToBigDecimal(orderMessaggeMap.get("capacityId")));
        omstruckOrder.setOrderNumber(orderMessaggeMap.get("orderNumber") + "-1");
        omstruckOrder.setOrderIssueTime(new Date());
        omstruckOrder.setOrderReceiveRefuseTime(new Date());
        omstruckOrder.setInsertTime(new Date());
        omstruckOrder.setInsertUsername("admin");
        omstruckOrder.setUnloadPointId(unloadPointId);
        omstruckOrderMapper.insertSelective(omstruckOrder);
        //添加运输订单子表
        Map<String, Object> mesMap = new HashMap<>();
        mesMap.put("orderId", newOrderId);
        omstruckOrderService.addOrderMaterial(orderMap, newOrderId);
        //添加总实绩
        Integer totalId = tmsTruckFeign.addTotalResult(mesMap);
        // 添加各种实绩
        int result = addAllSonResult(totalId, (Integer) map.get("lineId"));
        // 推送消息给司机
        Map<String,Object> newMap = new HashMap<>();
        newMap.put("orderId",orderId);
        newMap.put("newOrderId",newOrderId);
        pushMesToWebsocket(newMap);
        return result;
    }

    /**
     * 发送消息到 websocket 推送消息
     * orderId  订单Id  （unloadPointId）卸货点Id
     * @return
     */
    public void pushMesToWebsocket(Map<String, Object> map) {
        Map<String, Object> mesMap = omstruckOrderMapper.pushMesToWebsocket(map);
        BigDecimal newOrderId = (BigDecimal) map.get("newOrderId");
        if (newOrderId != null) {
            String orderNumber = (String) mesMap.get("orderNumber");
            String newOrderNumber = orderNumber + "-1";
            mesMap.remove("orderNumber");
            mesMap.put("orderNumber",newOrderNumber);
        }
        //添加消息实体
        HashMap<Object, Object> mapp = new HashMap<>();
        mapp.put("messageType", 4);
        mapp.put("sendPerson", "system");
        mapp.put("receivePerson", mesMap.get("capacityNumber"));
        mapp.put("messageContent", mesMap);
        mapp.put("createTime", new Date());
        ArrayList<Object> list = new ArrayList<>();
        list.add(mapp);
        //调用websocket接口推送
        HashMap<Object, Object> sendMap = new HashMap<>();
        //将消息实体放入list中存到map中
        sendMap.put("messages", list);
        String s = imFeign.sendToUser(sendMap);
        System.out.println(s);
    }

    /**
     * 通过运单id查询所有实绩点
     * @param orderId
     * @return
     */
    public List<Map<String,Object>> getOrderResult(BigDecimal orderId) {
        List<Map<String,Object>> seqmentList = omstruckOrderSeparateMapper.getSegmentList(orderId);
        Map<String,Object> resultPointList = omstruckOrderSeparateMapper.getResultPointList(orderId);
        if (seqmentList != null) {
        for (Map<String,Object> seqmentMap : seqmentList) {
            String linkName = (String) seqmentMap.get("linkName");
            if (resultPointList.size() != 0 && resultPointList != null) {
                if (linkName.equals("进厂")) {
                    String pointName = (String) resultPointList.get("gatepostNameEnter");
                    Date pointDate = (Date) resultPointList.get("resultEntryGateTime");
                    seqmentMap.put("pointName", pointName);
                    seqmentMap.put("pointDate", pointDate);
                }
                if (linkName.equals("计毛")) {
                    String pointName = (String) resultPointList.get("truckCalculateNumberM");
                    Date pointDate = (Date) resultPointList.get("resultGrossWeightTime");
                    seqmentMap.put("pointName", pointName);
                    seqmentMap.put("pointDate", pointDate);
                }
                if (linkName.equals("卸货")) {
                    String pointName = (String) resultPointList.get("warehouseNameUnLoad");
                    Date pointDate = (Date) resultPointList.get("resultEndTime");
                    seqmentMap.put("pointName", pointName);
                    seqmentMap.put("pointDate", pointDate);
                }
                if (linkName.equals("计皮")) {
                    String pointName = (String) resultPointList.get("truckCalculateNumberP");
                    Date pointDate = (Date) resultPointList.get("resultTareWeightTime");
                    seqmentMap.put("pointName", pointName);
                    seqmentMap.put("pointDate", pointDate);
                }
                if (linkName.equals("装货")) {
                    String pointName = (String) resultPointList.get("warehouseNameLoad");
                    Date pointDate = (Date) resultPointList.get("resultLoadEndTime");
                    seqmentMap.put("pointName", pointName);
                    seqmentMap.put("pointDate", pointDate);
                }
                if (linkName.equals("出厂")) {
                    String pointName = (String) resultPointList.get("gatepostNameLeave");
                    Date pointDate = (Date) resultPointList.get("resultOutGateTime");
                    seqmentMap.put("pointName", pointName);
                    seqmentMap.put("pointDate", pointDate);
                }
                if (linkName.equals("退货")) {
                    Date pointDate = (Date) resultPointList.get("returnTime");
                    seqmentMap.put("pointDate", pointDate);
                    seqmentMap.put("pointName", "");
                }
            }
        }
        }
        return seqmentList;
    }

    /**
     * 通过运输订单号查询实绩位置
     * @param orderNumber
     * @return
     */
    public List<Map<String, Object>> getOrderNumberResult(String orderNumber) {
        String substring = orderNumber.substring(0, orderNumber.length() - 1);
        BigDecimal orderId = omstruckOrderSeparateMapper.getOrderIdByOrderNumber(substring);
        List<Map<String, Object>> orderResult = getOrderResult(orderId);
        return orderResult;
    }
}
